/*
 	File:		Level2Glyphs.c
 
 	Contains:	QuickDraw GX to PostScript conversion code.
 						Routines for processing glyph runs using level-2 xyshow, xshow, and yshow.
					 	called from TextStylePrimitives.c
 
 	Version:	Technology:	Quickdraw GX 1.1.x
 	 
 	Copyright:	 1991-1997 by Apple Computer, Inc., all rights reserved.
*/

#include "PublicPostScriptIE.h"
#include "Private.h"
#include "PSIEResources.h"
#include <GXGraphics.h>

#ifdef resumeLabel
	#undef resumeLabel
#endif
#define resumeLabel(exception)




/*******************************************
	ProcessGlyphGroupPosL2:
	
	Routine translates a glyph group with advance
	bits specifying absolute positions for the glyphs
	to PostScript.
	
	nGlyphs:			How many glyphs in the group.
	positions:		Array of positions for the glyphs.
	text8:				Pointer to the 8 bit data for the group.
	byteCount:		Number of bytes in the 8-bit data.
	rdParams:			pointer to the RDUtil parameter block.
	nextDelta:		the delta of the first glyph after the glyph group passed in.
	
*********************************************/
OSErr ProcessGlyphGroupPosL2(TIEGlobalsHdl hIEGlobals, short nGlyphs, gxPoint *positions, char *text8, long byteCount,
															TRDParams *rdParams, TGlyphPosBias theBias, long addToIndex, gxPoint *nextDelta)
	{
		OSErr						status;
		gxPoint					*pPoint;
		gxPoint					*pPreviousPoint;
		Boolean					fixCurrPoint;
		
		register short	j;

		/* Put the pen at the position of the first glyph in the group */

		pPreviousPoint = &(positions[0]);
		nrequire(status = DoMoveto( rdParams, pPreviousPoint), failed_Output);

		if (nGlyphs > 1) {
			
			/*****
				Output all of the positions in a PostScript array,
				xyshow takes deltas from origin of last character.
			*****/		
			pPoint = &(positions[1]);		// point to second glyph to compute delta from previous.
	
			nrequire(status = DoBeginArray(hIEGlobals), failed_Output);
			
			if (theBias == eAllHorizontal) {
	
				rdParams->resIndex = kDoFixed;
				for (j = nGlyphs-1; j >= 0; --j) {
						
					status = RDResPrintf(rdParams, pPoint->x - pPreviousPoint->x);
					nrequire(status, failed_Output);
					pPreviousPoint = pPoint++;
						
				}//end for
											
				rdParams->resIndex = addToIndex +  kXshow;
				
			} else if (theBias == eAllVertical) {
			
				rdParams->resIndex = kDoFixed;
				for (j = nGlyphs-1; j >= 0; --j) {
						
					status = RDResPrintf(rdParams, pPoint->y - pPreviousPoint->y);
					nrequire(status, failed_Output);
					pPreviousPoint = pPoint++;
						
				}//end for
							
				rdParams->resIndex = addToIndex +  kYshow;
			
			} else {
				
				gxPoint deltaPoint;
			
				rdParams->resIndex = kDoPoint;
				for (j = nGlyphs-1; j >= 0; --j) {
					
					deltaPoint.x = pPoint->x - pPreviousPoint->x;
					deltaPoint.y = pPoint->y - pPreviousPoint->y;
					status = RDResPrintf(rdParams, &deltaPoint);
					nrequire(status, failed_Output);
					pPreviousPoint = pPoint++;
						
				}//end for
							
				rdParams->resIndex = addToIndex +  kXYshow;
			
			} //end if
			
			nrequire(status = DoEndArray(hIEGlobals), failed_Output);
			
			fixCurrPoint = true;				// x(y)show left currentpoint with delta of next glyph added in but we don't want it.
			
		} else {
		
			/** If there was only one character then just do a normal show **/
		
			rdParams->resIndex = addToIndex +  kShow;
			
			fixCurrPoint = false;				// show leaves current point correct for next group
			
		}//end if
			
		status = RDResPrintf(rdParams, text8, byteCount);
		nrequire(status, failed_Output);
		
		/** If the current point was overzealously updated, adjust it with an rmoveto **/
		
		if (fixCurrPoint && ( (nextDelta->x != 0) || (nextDelta->y != 0) ) ) {

			gxPoint deltaValue;;

			/** Negate the delta, because it was added in **/

			deltaValue.x = -nextDelta->x;
			deltaValue.y = -nextDelta->y;

			rdParams->resIndex = kRmoveto;
			status = RDResPrintf(rdParams, &deltaValue);
			nrequire(status, failed_Output);
		
		}//end if
				
failed_Output:	
		return(status);

	}//ProcessGlyphGroupPosL2

